"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventQueue = exports.EventTypes = void 0;
const Request_1 = require("./Request");
const utils_1 = require("./utils");
exports.EventTypes = {
    variableEvaluated: 'variableEvaluated',
    variableDefaulted: 'variableDefaulted'
};
class EventQueue {
    constructor(environmentKey, dvcClient, flushEventsMS) {
        this.environmentKey = environmentKey;
        this.client = dvcClient;
        this.eventQueue = [];
        this.aggregateEventMap = {};
        this.flushEventsMS = flushEventsMS || 10 * 1000;
        this.flushInterval = setInterval(this.flushEvents.bind(this), this.flushEventsMS);
    }
    flushEvents() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.client.config) {
                console.log('DVC Client not initialized to flush events!');
                return;
            }
            const eventsToFlush = [...this.eventQueue];
            const aggregateEventsToFlush = this.eventsFromAggregateEventMap();
            eventsToFlush.push(...aggregateEventsToFlush);
            if (!eventsToFlush.length) {
                return;
            }
            console.log(`DVC Flush ${eventsToFlush.length} Events`);
            this.eventQueue = [];
            this.aggregateEventMap = {};
            try {
                const res = yield (0, Request_1.publishEvents)(this.environmentKey, this.client.config, this.client.user, eventsToFlush);
                if (res.status !== 201) {
                    this.eventQueue.push(...eventsToFlush);
                }
                else {
                    console.log(`DVC Flushed ${eventsToFlush.length} Events.`);
                }
            }
            catch (ex) {
                this.client.eventEmitter.emitError(ex);
                this.eventQueue.push(...eventsToFlush);
            }
        });
    }
    /**
     * Queue DVCAPIEvent for producing
     */
    queueEvent(event) {
        this.eventQueue.push(event);
    }
    /**
     * Queue DVCEvent that can be aggregated together, where multiple calls are aggregated
     * by incrementing the 'value' field.
     */
    queueAggregateEvent(event) {
        (0, utils_1.checkParamDefined)('type', event.type);
        (0, utils_1.checkParamDefined)('target', event.target);
        event.date = Date.now();
        event.value = 1;
        const aggEventType = this.aggregateEventMap[event.type];
        if (!aggEventType) {
            this.aggregateEventMap[event.type] = { [event.target]: event };
        }
        else if (aggEventType[event.target]) {
            aggEventType[event.target].value++;
        }
        else {
            aggEventType[event.target] = event;
        }
    }
    /**
     * Turn the Aggregate Event Map into an Array of DVCAPIEvent objects for publishing.
     */
    eventsFromAggregateEventMap() {
        return Object.values(this.aggregateEventMap).map((typeMap) => Object.values(typeMap)).flat();
    }
}
exports.EventQueue = EventQueue;
//# sourceMappingURL=EventQueue.js.map