"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.publishEvents = exports.post = exports.getConfigJson = exports.get = exports.baseRequestHeaders = exports.EVENTS_PATH = exports.CONFIG_PATH = exports.EVENT_URL = exports.BASE_URL = exports.HOST = void 0;
const utils_1 = require("./utils");
const axios_1 = __importDefault(require("axios"));
const axios_retry_1 = __importDefault(require("axios-retry"));
const axiosClient = axios_1.default.create({
    validateStatus: (status) => status < 400 && status >= 200,
});
(0, axios_retry_1.default)(axiosClient, {
    retries: 5,
    retryDelay: (retryCount) => {
        return Math.pow(2, retryCount) * 1000;
    },
    retryCondition: (error) => {
        var _a;
        return (((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) || 0) >= 500;
    }
});
exports.HOST = '.devcycle.com';
exports.BASE_URL = 'https://sdk-api';
exports.EVENT_URL = 'https://events';
exports.CONFIG_PATH = '/v1/sdkConfig';
exports.EVENTS_PATH = '/v1/events';
const baseRequestHeaders = (environmentKey) => {
    return Object.assign({ 'Content-Type': 'application/json' }, (environmentKey ? { 'Authorization': environmentKey } : {}));
};
exports.baseRequestHeaders = baseRequestHeaders;
const get = (url) => __awaiter(void 0, void 0, void 0, function* () {
    return yield axiosClient.request({
        method: 'GET',
        url: `${url}`,
        headers: (0, exports.baseRequestHeaders)()
    });
});
exports.get = get;
const getConfigJson = (environmentKey, user) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const queryParams = `${(0, utils_1.serializeUser)(user)}`;
    const url = `${exports.BASE_URL}${exports.HOST}${exports.CONFIG_PATH}?envKey=${environmentKey}${queryParams && '&' + queryParams}`;
    try {
        const res = yield (0, exports.get)(url);
        return res.data;
    }
    catch (ex) {
        console.error(`Request to get config failed for url: ${url}, ` +
            `response message: ${ex.message}, response data: ${(_a = ex === null || ex === void 0 ? void 0 : ex.response) === null || _a === void 0 ? void 0 : _a.data}`);
        throw new Error('Failed to download DevCycle config.');
    }
});
exports.getConfigJson = getConfigJson;
const post = (url, environmentKey, body) => __awaiter(void 0, void 0, void 0, function* () {
    return yield axiosClient.request({
        method: 'POST',
        url,
        data: body,
        headers: (0, exports.baseRequestHeaders)(environmentKey)
    });
});
exports.post = post;
const publishEvents = (envKey, config, user, events) => __awaiter(void 0, void 0, void 0, function* () {
    if (!envKey) {
        throw new Error('Missing envKey to publish events to Events API');
    }
    const payload = (0, utils_1.generateEventPayload)(config, user, events);
    console.log(`Submit Events Payload: ${JSON.stringify(payload)}`);
    const res = yield (0, exports.post)(`${exports.EVENT_URL}${exports.HOST}${exports.EVENTS_PATH}`, envKey, payload);
    if (res.status !== 201) {
        console.error(`Error posting events, status: ${res.status}, body: ${res.data}`);
    }
    else {
        console.log(`Posted Events, status: ${res.status}, body: ${res.data}`);
    }
    return res;
});
exports.publishEvents = publishEvents;
exports.default = {
    get: exports.get,
    post: exports.post,
    getConfigJson: exports.getConfigJson,
    publishEvents: exports.publishEvents
};
//# sourceMappingURL=Request.js.map