"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateEventPayload = exports.checkParamType = exports.checkParamDefined = exports.serializeUser = void 0;
const RequestEvent_1 = require("./RequestEvent");
const serializeUser = (user) => {
    const convertToQueryFriendlyFormat = (property) => {
        if (property instanceof Date) {
            return property.getTime();
        }
        if (typeof property === 'object') {
            return JSON.stringify(property);
        }
        return property;
    };
    return Object.keys(user)
        .reduce((prev, curr, _) => {
        const userProperty = convertToQueryFriendlyFormat(user[curr]);
        const nextQueryParam = userProperty !== null && userProperty !== undefined
            ? `${prev && '&'}${curr}=${encodeURIComponent(userProperty)}`
            : '';
        return `${prev}${nextQueryParam}`;
    }, '');
};
exports.serializeUser = serializeUser;
const checkParamDefined = (name, param) => {
    if (param === undefined || param === null) {
        throw new Error(`Missing parameter: ${name}`);
    }
};
exports.checkParamDefined = checkParamDefined;
const checkParamType = (name, param, type) => {
    if (!param) {
        throw new Error(`Missing parameter: ${name}`);
    }
    if (typeof param !== type) {
        throw new Error(`${name} is not of type ${type}`);
    }
};
exports.checkParamType = checkParamType;
function generateEventPayload(config, user, events) {
    return {
        events: events.map((event) => {
            return new RequestEvent_1.DVCRequestEvent(event, user.user_id, config.featureVariationMap);
        }),
        user
    };
}
exports.generateEventPayload = generateEventPayload;
exports.default = {
    serializeUser: exports.serializeUser,
    checkParamDefined: exports.checkParamDefined,
    generateEventPayload
};
//# sourceMappingURL=utils.js.map